"""
dolbyio_rest_apis.streaming.models.publish_token
~~~~~~~~~~~~~~~

This module contains the models used by the Publish Token models.
"""

from typing import List
from dolbyio_rest_apis.core.helpers import get_value_or_default

class PublishTokenStream(dict):
    """The :class:`PublishTokenStream` object, which represents a Publish Token Stream."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.stream_name = get_value_or_default(self, 'streamName', None)
        self.is_regex = get_value_or_default(self, 'isRegex', False)

class PublishToken(dict):
    """The :class:`PublishToken` object, which represents a Publish Token."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.id = get_value_or_default(self, 'id', None)
        self.label = get_value_or_default(self, 'label', None)
        self.token = get_value_or_default(self, 'token', None)
        self.added_on = get_value_or_default(self, 'addedOn', None)
        self.expires_on = get_value_or_default(self, 'expiresOn', None)
        self.is_active = get_value_or_default(self, 'isActive', None)
        self.streams = []
        for stream in self['streams']:
            self.streams.append(PublishTokenStream(stream))
        self.allowed_origins = get_value_or_default(self, 'allowedOrigins', None)
        self.allowed_ip_addresses = get_value_or_default(self, 'allowedIpAddresses', None)
        self.bind_ips_on_usage = get_value_or_default(self, 'bindIpsOnUsage', None)
        self.allowed_countries = get_value_or_default(self, 'allowedCountries', None)
        self.denied_countries = get_value_or_default(self, 'deniedCountries', None)
        self.origin_cluster = get_value_or_default(self, 'originCluster', None)
        self.subscribe_requires_auth = get_value_or_default(self, 'subscribeRequiresAuth', False)
        self.record = get_value_or_default(self, 'record', False)
        self.multisource = get_value_or_default(self, 'multisource', False)

class CreateUpdatePublishTokenStream():
    """The :class:`CreateUpdatePublishTokenStream` object, which represents a Publish Token Stream."""

    def __init__(self, stream_name: str, is_regex: bool):
        self.stream_name = stream_name
        self.is_regex = is_regex

class UpdatePublishToken():
    """The :class:`UpdatePublishToken` object, which represents an Update Publish Token request."""

    def __init__(self):
        self.label: str | None = None
        self.refresh_token: bool | None = None
        self.is_active: bool | None = None
        self.add_token_streams: List[CreateUpdatePublishTokenStream] | None = None
        self.remove_token_streams: List[CreateUpdatePublishTokenStream] | None = None
        self.update_allowed_origins: List[str] | None = None
        self.update_allowed_ip_addresses: List[str] | None = None
        self.update_bind_ips_on_usage: int | None = None
        self.update_allowed_countries: List[str] | None = None
        self.update_denied_countries: List[str] | None = None
        self.update_origin_cluster: str | None = None
        self.subscribe_requires_auth: bool | None = None
        self.record: bool | None = None
        self.multisource: bool | None = None

class CreatePublishToken():
    """The :class:`CreatePublishToken` object, which represents a Create Publish Token request."""

    def __init__(self, label: str):
        self.label = label
        self.expires_on: str | None = None
        self.streams: List[CreateUpdatePublishTokenStream] = []
        self.allowed_origins: List[str] | None = None
        self.allowed_ip_addresses: List[str] | None = None
        self.bind_ips_on_usage: int | None = None
        self.allowed_countries: List[str] | None = None
        self.denied_countries: List[str] | None = None
        self.origin_cluster: str | None = None
        self.subscribe_requires_auth: bool = False
        self.record: bool = False
        self.multisource: bool = False

class ActivePublishToken(dict):
    """The :class:`ActivePublishToken` object."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.token_ids: List[int] = []
        for tid in self['tokenIds']:
            self.token_ids.append(tid)

class FailedToken(dict):
    """The :class:`FailedToken` object, which represents a Failed Token."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.token_id = get_value_or_default(self, 'tokenId', None)
        self.error_message = get_value_or_default(self, 'errorMessage', None)

class DisablePublishTokenResponse(dict):
    """The :class:`DisablePublishTokenResponse` object, which represents a the response to the disable publish token."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.successful_tokens = get_value_or_default(self, 'successfulTokens', None)
        self.failed_tokens: List[int] = []
        for ft in self['failedTokens']:
            self.failed_tokens.append(FailedToken(ft))



"""
dolbyio_rest_apis_cli.media.cli
~~~~~~~~~~~~~~~

This module contains the main entry point for the CLI.

"""

import argparse
import asyncio
from dolbyio_rest_apis_cli.core import version
from dolbyio_rest_apis_cli.media.commands import authentication, enhance, io, jobs, mastering, transcode

async def main():
    parser = argparse.ArgumentParser(description='Dolby.io REST APIs CLI - Media')
    sub_parsers = parser.add_subparsers(dest='command')

    version.add_arguments(parser)
    authentication.add_arguments(sub_parsers)
    enhance.add_arguments(sub_parsers)
    io.add_arguments(sub_parsers)
    jobs.add_arguments(sub_parsers)
    mastering.add_arguments(sub_parsers)
    transcode.add_arguments(sub_parsers)

    args = parser.parse_args()

    if args.version:
        version.execute_command()
    elif args.command is None:
        parser.print_help()
    elif args.command == authentication.command_name():
        await authentication.execute_command(args)
    elif args.command == enhance.command_name():
        await enhance.execute_command(args)
    elif args.command == io.command_name():
        await io.execute_command(args)
    elif args.command == jobs.command_name():
        await jobs.execute_command(args)
    elif args.command == mastering.command_name():
        await mastering.execute_command(args)
    elif args.command == transcode.command_name():
        await transcode.execute_command(args)

def cli():
    asyncio.get_event_loop().run_until_complete(main())

if __name__ == '__main__':
    cli()



"""
dolbyio_rest_apis.core.helpers
~~~~~~~~~~~~~~~

This module contains some helpers.
"""

import enum

def get_value(value):
    return value.value if isinstance(value, enum.Enum) else value

def get_value_or_default(obj, name: str, default_value):
    return obj[name] if name in obj else default_value

def add_if_not_none(array, name: str, value):
    if not value is None:
        array[name] = get_value(value)

def in_and_not_none(dictionary, key):
    return key in dictionary and not dictionary[key] is None



"""
dolbyio_rest_apis.media.models.result_error
~~~~~~~~~~~~~~~

This module contains the Result Error model.
"""

from dolbyio_rest_apis.core.helpers import get_value_or_default

class ResultError(dict):
    """The :class:`ResultError` object, which represents an enhance job error."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.type = get_value_or_default(self, 'type', None)
        self.title = get_value_or_default(self, 'title', None)
        self.detail = get_value_or_default(self, 'detail', None)



"""
dolbyio_rest_apis.communications.models
~~~~~~~~~~~~~~~

This module contains the models used by the Dolby.io APIs.
"""

from enum import Enum
from dataclasses import dataclass
from typing import List
from dolbyio_rest_apis.core.helpers import get_value_or_default, in_and_not_none

class Participant:
    """The :class:`Participant` object, which represents a participant's permissions."""

    def __init__(self, external_id: str, permissions, notify: bool):
        self.external_id = external_id
        self.permissions = permissions
        self.notify = notify

class RTCPMode(str, Enum):
    """The :class:`RTCPMode` enumeration, which represents the possible RTCP modes."""

    WORST = 'worst'
    '''Adjusts the transmission bitrate to the receiver who has the worst network conditions.'''

    AVERAGE = 'average'
    '''Averages the available bandwidth of all the receivers and adjusts the transmission bitrate to this value.'''

    MAX = 'max'
    '''Does not adjust the transmission bitrate to the receiver’s bandwidth.'''

class VideoCodec(str, Enum):
    """The :class:`VideoCodec` enumeration, which represents the possible video codecs."""

    VP8 = 'VP8'
    H264 = 'H264'

class Permission(str, Enum):
    """The :class:`Permission` enumeration, which represents the possible participant's permissions."""

    INVITE = 'INVITE'
    '''Allows a participant to invite participants to a conference.'''

    JOIN = 'JOIN'
    '''Allows a participant to join a conference.'''

    SEND_AUDIO = 'SEND_AUDIO'
    '''Allows a participant to send an audio stream during a conference.'''

    SEND_VIDEO = 'SEND_VIDEO'
    '''Allows a participant to send a video stream during a conference.'''

    SHARE_SCREEN = 'SHARE_SCREEN'
    '''Allows a participant to share their screen during a conference.'''

    SHARE_VIDEO = 'SHARE_VIDEO'
    '''Allows a participant to share a video during a conference.'''

    SHARE_FILE = 'SHARE_FILE'
    '''Allows a participant to share a file during a conference.'''

    SEND_MESSAGE = 'SEND_MESSAGE'
    '''Allows a participant to send a message to other participants during a conference.'''

    RECORD = 'RECORD'
    '''Allows a participant to record a conference.'''

    STREAM = 'STREAM'
    '''Allows a participant to stream a conference.'''

    KICK = 'KICK'
    '''Allows a participant to kick other participants from a conference.'''

    UPDATE_PERMISSIONS = 'UPDATE_PERMISSIONS'
    '''Allows a participant to update other participants' permissions.'''

@dataclass
class UserToken:
    """Representation of a User access token."""
    external_id: str
    token: str

class Conference(dict):
    """Representation of a newly created Conference."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.conference_id = get_value_or_default(self, 'conferenceId', None)
        self.conference_alias = get_value_or_default(self, 'conferenceAlias', None)
        self.conference_pincode = get_value_or_default(self, 'conferencePincode', None)
        self.is_protected = get_value_or_default(self, 'isProtected', False)
        self.owner_token = get_value_or_default(self, 'ownerToken', None)

        self.user_tokens: List[UserToken] = []
        if in_and_not_none(self, 'usersTokens'):
            for key in self['usersTokens'].keys():
                user_token = UserToken(key, self['usersTokens'][key])
                self.user_tokens.append(user_token)

class RemixStatus(dict):
    """Representation of a Remix Status."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.status = get_value_or_default(self, 'status', None)
        self.region = get_value_or_default(self, 'region', None)
        self.alias = get_value_or_default(self, 'alias', None)

@dataclass
class Coordinates:
    """Representation of a Coordinate object."""
    x: int
    y: int
    z: int

@dataclass
class SpatialAudioEnvironment:
    """
    The spatial environment of an application,
    so the audio renderer understands which directions the application considers
    forward, up, and right and which units it uses for distance.
    """
    scale: Coordinates
    forward: Coordinates
    up: Coordinates
    right: Coordinates

@dataclass
class SpatialAudioListener:
    """Representation of the listener's audio position and direction, defined using Cartesian coordinates."""
    position: Coordinates
    direction: Coordinates

@dataclass
class SpatialAudioUser:
    """Representation of the user's position, defined using Cartesian coordinates."""
    external_id: str
    position: Coordinates



"""
dolbyio_rest_apis.media.models.job_response
~~~~~~~~~~~~~~~

This module contains the Job response base model
"""

from dolbyio_rest_apis.core.helpers import get_value_or_default
from .result_error import ResultError

class JobResponse(dict):
    """The :class:`JobResult` object, which represents the result for a job operation."""

    def __init__(self, job_id, dictionary: dict):
        dict.__init__(self, dictionary)

        self.job_id = job_id
        self.api_version = get_value_or_default(dictionary, 'api_version', None)
        self.path = get_value_or_default(dictionary, 'path', None)
        self.status = get_value_or_default(dictionary, 'status', None)
        self.progress = get_value_or_default(dictionary, 'progress', 0)
        self.result = None

        if 'error' in dictionary:
            self.error = ResultError(dictionary['error'])

